# Generated by Django 4.2.15 on 2024-09-25 20:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0221_enterprisegroup_group_type_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisegroup',
            name='group_type',
            field=models.CharField(choices=[('budget', 'Budget'), ('flex', 'Flex')], default='flex', help_text='The type of enterprise group', max_length=20, verbose_name='Group Type'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisegroup',
            name='group_type',
            field=models.CharField(choices=[('budget', 'Budget'), ('flex', 'Flex')], default='flex', help_text='The type of enterprise group', max_length=20, verbose_name='Group Type'),
        ),
    ]



# Generated by Django 3.2.15 on 2022-09-28 16:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0015_auto_20220718_2113'),
    ]

    operations = [
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='marked_for',
            field=models.CharField(blank=True, help_text='Flag marking a record as needing a form of transmitting', max_length=32, null=True),
        ),
    ]



# Generated by Django 3.2.19 on 2023-07-12 17:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0175_auto_20230629_2330'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='auth_org_id',
            field=models.CharField(blank=True, help_text="Enterprise customer's authentication ID ", max_length=80, null=True),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='auth_org_id',
            field=models.CharField(blank=True, help_text="Enterprise customer's authentication ID ", max_length=80, null=True),
        ),
    ]



# Generated by Django 2.2.20 on 2021-06-29 21:29

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import simple_history.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('enterprise', '0135_adminnotification_adminnotificationfilter_adminnotificationread'),
    ]

    operations = [
        migrations.AlterField(
            model_name='systemwideenterpriseuserroleassignment',
            name='role',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='system_wide_role_assignments', to='enterprise.SystemWideEnterpriseRole'),
        ),
        migrations.CreateModel(
            name='HistoricalSystemWideEnterpriseUserRoleAssignment',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('applies_to_all_contexts', models.BooleanField(default=False, help_text='If true, indicates that the user is effectively assigned their role for any and all contexts. Defaults to False.')),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('enterprise_customer', models.ForeignKey(blank=True, db_constraint=False, help_text='The EnterpriseCustomer for which the role is assigned for the user.  If creating a new assignment, you must first provide a user email and role, then click "Save and continue editing" BEFORE selecting from this dropdown.', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.EnterpriseCustomer')),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('role', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.SystemWideEnterpriseRole')),
                ('user', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical system wide enterprise user role assignment',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='HistoricalEnterpriseCustomerUser',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('user_id', models.PositiveIntegerField(db_index=True)),
                ('active', models.BooleanField(default=True)),
                ('linked', models.BooleanField(default=True)),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('enterprise_customer', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.EnterpriseCustomer')),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical Enterprise Customer Learner',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



"""
URL definitions for Canvas API.
"""

from django.urls import path

from integrated_channels.canvas.views import CanvasCompleteOAuthView

urlpatterns = [
    path('oauth-complete', CanvasCompleteOAuthView.as_view(),
         name='canvas-oauth-complete'
         ),
]



"""
    url mappings for integrated_channels/v1/blackboard/
"""

from rest_framework import routers

from .views import BlackboardConfigurationViewSet, BlackboardGlobalConfigurationViewSet

app_name = 'blackboard'
router = routers.DefaultRouter()
router.register(r'configuration', BlackboardConfigurationViewSet, basename="configuration")
router.register(r'global-configuration', BlackboardGlobalConfigurationViewSet, basename="global-configuration")
urlpatterns = router.urls



# Generated by Django 4.2.10 on 2024-06-14 08:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0213_populate_learner_portal_sidebar_content_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='learner_portal_sidebar_content',
            field=models.TextField(blank=True, help_text='Text shown on the learner portal dashboard for customer specific purposes. Open HTML field.'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='learner_portal_sidebar_content',
            field=models.TextField(blank=True, help_text='Text shown on the learner portal dashboard for customer specific purposes. Open HTML field.'),
        ),
    ]



# Generated by Django 3.2.23 on 2024-04-09 11:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('degreed2', '0027_auto_20240329_1537'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='client_id',
        ),
        migrations.RemoveField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='client_secret',
        ),
    ]



# Generated by Django 3.2.23 on 2024-06-04 10:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('blackboard', '0022_blackboardlearnerassessmentdatatransmissionaudit_is_transmitted_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='blackboardenterprisecustomerconfiguration',
            name='client_id',
        ),
        migrations.RemoveField(
            model_name='blackboardenterprisecustomerconfiguration',
            name='client_secret',
        ),
    ]



"""
URL definitions for integrated_channels API endpoint.
"""

from django.urls import include, path

app_name = 'api'
urlpatterns = [
    path('v1/', include('integrated_channels.api.v1.urls'), name='api')
]



"""
Views containing APIs for Blackboard integrated channel
"""

import base64
import logging
from http import HTTPStatus
from urllib.parse import urljoin

import requests
from rest_framework import generics
from rest_framework.exceptions import NotFound
from rest_framework.renderers import JSONRenderer

from django.apps import apps
from django.conf import settings
from django.core.exceptions import ValidationError
from django.shortcuts import render

from enterprise.utils import get_enterprise_customer
from integrated_channels.blackboard.models import BlackboardEnterpriseCustomerConfiguration

LOGGER = logging.getLogger(__name__)


def log_auth_response(auth_token_url, data):
    """
    Logs the response from a refresh_token fetch.
    some fields may be absent in the response:
    ref: https://www.oauth.com/oauth2-servers/access-tokens/access-token-response/
    """
    scope = data['scope'] if 'scope' in data else 'not_found'
    user_id = data['user_id'] if 'user_id' in data else 'not_found'
    LOGGER.info("BLACKBOARD: response from {} contained: token_type={},"
                "expires_in={}, scope={}, user_id={}".format(
                    auth_token_url,
                    data['token_type'],
                    data['expires_in'],
                    scope,
                    user_id,
                ))


class BlackboardCompleteOAuthView(generics.ListAPIView):
    """
        **Use Cases**

            Retrieve and save a Blackboard OAuth refresh token after an enterprise customer
            authorizes to integrated courses. Typically for use to plug into the redirect_uri field
            in visiting the 'authorizationcode' endpoint:
            Ref: https://developer.blackboard.com/portal/displayApi/Learn
            e.g. https://blackboard.edx.us.org/learn/api/public/v1/oauth2/
                 authorizationcode?redirect_uri={{this_endpoint}}&response_type=code
                 &client_id={{app id}}&state={{blackboard_enterprise_customer_configuration.uuid}}

        **Example Requests**

            GET /blackboard/oauth-complete?code=123abc&state=abc123

        **Query Parameters for GET**

            * code: The one time use string generated by the Blackboard API used to fetch the
            access and refresh tokens for integrating with Blackboard.

            * state: The user's enterprise customer uuid used to associate the incoming
            code with an enterprise configuration model.

        **Response Values**

            HTTP 200 "OK" if successful

            HTTP 400 if code/state is not provided

            HTTP 404 if state is not valid or contained in the set of registered enterprises

    """
    renderer_classes = [JSONRenderer, ]

    def render_page(self, request, error):
        """
        Return a success or failure page based on Blackboard OAuth response
        """
        success_template = 'enterprise/admin/oauth_authorization_successful.html'
        error_template = 'enterprise/admin/oauth_authorization_failed.html'
        template = error_template if error else success_template

        return render(request, template, context={})

    def get(self, request, *args, **kwargs):
        app_config = apps.get_app_config('blackboard')
        oauth_token_path = app_config.oauth_token_auth_path

        # Check if encountered an error when generating the oauth code.
        request_error = request.GET.get('error')
        if request_error:
            LOGGER.error(
                "Blackboard OAuth API encountered an error when generating client code - "
                "error: {} description: {}".format(
                    request_error,
                    request.GET.get('error_description')
                )
            )
            return self.render_page(request, 'error')

        # Retrieve the newly generated code and state (Enterprise user's ID)
        client_code = request.GET.get('code')
        state_uuid = request.GET.get('state')

        if not state_uuid:
            LOGGER.error("Blackboard Configuration uuid (as 'state' url param) needed to obtain refresh token")
            return self.render_page(request, 'error')

        if not client_code:
            LOGGER.error("'code' url param was not provided, needed to obtain refresh token")
            return self.render_page(request, 'error')

        try:
            enterprise_config = BlackboardEnterpriseCustomerConfiguration.objects.get(uuid=state_uuid)
        except (BlackboardEnterpriseCustomerConfiguration.DoesNotExist, ValidationError):
            enterprise_config = None

        # old urls may use the enterprise customer uuid in place of the config uuid, so lets fallback
        if not enterprise_config:
            enterprise_customer = get_enterprise_customer(state_uuid)

            if not enterprise_customer:
                LOGGER.error(f"No state data found for given uuid: {state_uuid}.")
                return self.render_page(request, 'error')

            try:
                enterprise_config = BlackboardEnterpriseCustomerConfiguration.objects.filter(
                    enterprise_customer=enterprise_customer
                ).first()
            except BlackboardEnterpriseCustomerConfiguration.DoesNotExist:
                LOGGER.error(f"No state data found for given uuid: {state_uuid}")
                return self.render_page(request, 'error')

        BlackboardGlobalConfiguration = apps.get_model(
            'blackboard',
            'BlackboardGlobalConfiguration'
        )
        blackboard_global_config = BlackboardGlobalConfiguration.current()
        if not blackboard_global_config:
            LOGGER.error("No global Blackboard configuration found")
            return self.render_page(request, 'error')

        auth_header = self._create_auth_header(enterprise_config, blackboard_global_config)

        access_token_request_params = {
            'grant_type': 'authorization_code',
            'redirect_uri': settings.LMS_INTERNAL_ROOT_URL + "/blackboard/oauth-complete",
            'code': client_code,
        }

        auth_token_url = urljoin(enterprise_config.blackboard_base_url, oauth_token_path)

        auth_response = requests.post(
            auth_token_url,
            access_token_request_params,
            headers={
                'Authorization': auth_header,
                'Content-Type': 'application/x-www-form-urlencoded'
            })

        try:
            data = auth_response.json()
            if 'refresh_token' not in data:
                LOGGER.error(
                    "BLACKBOARD: failed to find refresh_token in auth response. "
                    "Auth response text: {}, Response code: {}, JSON response: {}".format(
                        auth_response.text, auth_response.status_code, data))
                return self.render_page(request, 'error')

            log_auth_response(auth_token_url, data)
            refresh_token = data['refresh_token']
            if refresh_token.strip():
                enterprise_config.refresh_token = refresh_token
                enterprise_config.save()
            else:
                LOGGER.error("BLACKBOARD: Invalid/empty refresh_token! Cannot use it.")
                return self.render_page(request, 'error')
        except KeyError:
            LOGGER.error(
                "BLACKBOARD: failed to find required data in auth response. "
                "Auth response text: {}, Response code: {}, JSON response: {}".format(
                    auth_response.text, auth_response.status_code, data))
            return self.render_page(request, 'error')

        status = '' if auth_response.status_code == 200 else 'error'

        return self.render_page(request, status)

    def _create_auth_header(self, enterprise_config, blackboard_global_config):
        """
        Auth header in oauth2 token format as per Blackboard doc
        """
        app_key = enterprise_config.decrypted_client_id
        if not app_key:
            if not blackboard_global_config.app_key:
                raise NotFound(
                    "Failed to generate oauth access token: Client ID required.",
                    HTTPStatus.INTERNAL_SERVER_ERROR.value
                )
            app_key = blackboard_global_config.app_key
        app_secret = enterprise_config.decrypted_client_secret
        if not app_secret:
            if not blackboard_global_config.app_secret:
                raise NotFound(
                    "Failed to generate oauth access token: Client secret required.",
                    HTTPStatus.INTERNAL_SERVER_ERROR.value
                )
            app_secret = blackboard_global_config.app_secret
        return f"Basic {base64.b64encode(f'{app_key}:{app_secret}'.encode('utf-8')).decode()}"



# Generated by Django 2.2.13 on 2020-06-29 17:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0097_auto_20200619_1130'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecourseenrollment',
            name='saved_for_later',
            field=models.BooleanField(default=False, help_text='Specifies whether a user marked this course as saved for later in the learner portal.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecourseenrollment',
            name='saved_for_later',
            field=models.BooleanField(default=False, help_text='Specifies whether a user marked this course as saved for later in the learner portal.'),
        ),
    ]



# Generated by Django 2.2.24 on 2021-09-30 15:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0003_contentmetadataitemtransmission_content_last_changed'),
    ]

    operations = [
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='enterprise_customer_catalog_uuid',
            field=models.UUIDField(blank=True, help_text='The enterprise catalog that this metadata item was derived from', null=True),
        ),
    ]



# Generated by Django 3.2.12 on 2022-03-23 17:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('degreed2', '0008_degreed2learnerdatatransmissionaudit_course_completed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='degreed2learnerdatatransmissionaudit',
            name='completed_timestamp',
            field=models.CharField(blank=True, help_text='yyyy-mm-ddTHH:MM:SS format. Can be left unset for audit records.', max_length=19, null=True),
        ),
    ]



# Generated by Django 3.2.17 on 2023-02-21 18:38

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0168_auto_20230222_1621'),
    ]

    operations = [
        migrations.AlterField(
            model_name='licensedenterprisecourseenrollment',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, unique=True)
        ),
        migrations.AlterField(
            model_name='historicallearnercreditenterprisecourseenrollment',
            name='uuid',
            field=models.UUIDField(db_index=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='historicallicensedenterprisecourseenrollment',
            name='uuid',
            field=models.UUIDField(db_index=True, default=uuid.uuid4, editable=False),
        ),
        migrations.AlterField(
            model_name='learnercreditenterprisecourseenrollment',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
        ),
    ]



"""
Django admin integration for configuring moodle app to communicate with Moodle systems.
"""

from django_object_actions import DjangoObjectActions

from django import forms
from django.contrib import admin, messages
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect
from django.utils.translation import gettext_lazy as _

from integrated_channels.integrated_channel.admin import BaseLearnerDataTransmissionAuditAdmin
from integrated_channels.moodle.models import MoodleEnterpriseCustomerConfiguration, MoodleLearnerDataTransmissionAudit


class MoodleEnterpriseCustomerConfigurationForm(forms.ModelForm):
    """
    Django admin form for MoodleEnterpriseCustomerConfiguration.
    """
    class Meta:
        model = MoodleEnterpriseCustomerConfiguration
        fields = '__all__'

    def clean(self):
        cleaned_data = super().clean()
        cleaned_username = cleaned_data.get('decrypted_username')
        cleaned_password = cleaned_data.get('decrypted_password')
        cleaned_token = cleaned_data.get('decrypted_token')
        if cleaned_token and (cleaned_username or cleaned_password):
            raise ValidationError(_('Cannot set both a Username/Password and Token'))
        if (cleaned_username and not cleaned_password) or (cleaned_password and not cleaned_username):
            raise ValidationError(_('Must set both a Username and Password, not just one'))


@admin.register(MoodleEnterpriseCustomerConfiguration)
class MoodleEnterpriseCustomerConfigurationAdmin(DjangoObjectActions, admin.ModelAdmin):
    """
    Django admin model for MoodleEnterpriseCustomerConfiguration.
    """

    raw_id_fields = (
        'enterprise_customer',
    )

    form = MoodleEnterpriseCustomerConfigurationForm
    change_actions = ('force_content_metadata_transmission',)

    @admin.action(
        description="Force content metadata transmission for this Enterprise Customer"
    )
    def force_content_metadata_transmission(self, request, obj):
        """
        Updates the modified time of the customer record to retransmit courses metadata
        and redirects to configuration view with success or error message.
        """
        try:
            obj.enterprise_customer.save()
            messages.success(
                request,
                f'''The moodle enterprise customer content metadata
                “<MoodleEnterpriseCustomerConfiguration for Enterprise
                {obj.enterprise_customer.name}>” was updated successfully.''',
            )
        except ValidationError:
            messages.error(
                request,
                f'''The moodle enterprise customer content metadata
                “<MoodleEnterpriseCustomerConfiguration for Enterprise
                {obj.enterprise_customer.name}>” was not updated successfully.''',
            )
        return HttpResponseRedirect(
            "/admin/moodle/moodleenterprisecustomerconfiguration"
        )
    force_content_metadata_transmission.label = "Force content metadata transmission"


@admin.register(MoodleLearnerDataTransmissionAudit)
class MoodleLearnerDataTransmissionAuditAdmin(BaseLearnerDataTransmissionAuditAdmin):
    """
    Django admin model for MoodleLearnerDataTransmissionAudit.
    """
    list_display = (
        "enterprise_course_enrollment_id",
        "course_id",
        "status",
        "modified",
    )

    readonly_fields = (
        "moodle_user_email",
        "progress_status",
        "content_title",
        "enterprise_customer_name",
        "friendly_status_message",
        "api_record",
    )

    list_per_page = 1000

    class Meta:
        model = MoodleLearnerDataTransmissionAudit



# Generated by Django 3.2.23 on 2024-03-12 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0202_enterprisegroup_applies_to_all_contexts_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisegroupmembership',
            name='activated_at',
            field=models.DateTimeField(blank=True, default=None, help_text='The moment at which the membership record is written with an Enterprise Customer User record.', null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisegroupmembership',
            name='activated_at',
            field=models.DateTimeField(blank=True, default=None, help_text='The moment at which the membership record is written with an Enterprise Customer User record.', null=True),
        ),
    ]
